(ns sandbar.examples.user-form
  "Use sandbar to create a form."
  (:use [ring.adapter.jetty :only [run-jetty]]
        [ring.middleware.file :only [wrap-file]]
        [compojure.core :only [defroutes GET]]
        [sandbar.core :only [get-param]]
        [sandbar.stateful-session :only [wrap-stateful-session
                                         flash-put!]]
        [sandbar.validation :only [build-validator
                                   non-empty-string
                                   add-validation-error]])
  (:require [compojure.route :as route]
            [sandbar.forms :as forms]
            [sandbar.examples.database :as db]
            [sandbar.examples.views :as views]))

(def labels
     {:username "Username"
      :password "Password"
      :first-name "First Name"
      :last-name "Last Name"
      :email "Email Address"
      :roles "Assigned Roles"
      :user "Normal User"
      :admin "Administrator"
      :notes "Notes"
      :notes-validation-error "Notes are required."})

(defn password-strength [m]
  (if (< (count (:password m)) 10)
    (add-validation-error m :password "Passwords must have > 10 characters.")
    m))

(def user-form-validator
     (build-validator
      (non-empty-string :username :password labels)
      :ensure
      password-strength))

(forms/defform user-form "/user/edit"
  :fields [(forms/hidden :id)
           (forms/textfield :username)
           (forms/password :password)
           (forms/textfield :first-name :last-name :email)
           (forms/multi-checkbox :roles (db/all-roles) identity)]
  :load #(db/find-user %)
  :on-cancel "/"
  :on-success #(do
                 (db/store-user %)
                 (flash-put! :user-message "User has been saved.")
                 "/")
  :properties labels
  :validator user-form-validator
  :field-layout [1 1 2]
  :style :over-under
  :title #(case % :add "Add New User" "Edit User")
  :buttons [[:save] [:save-and-new "Save and New"] [:cancel]])

(forms/extend-form user-form :with edit-form
    :when (fn [action request form-data]
            (or (= action :edit)
                (get-param (-> request :params) :notes)))
    :fields [(forms/textarea :notes {:rows 5 :cols 70})]
    :validator #(non-empty-string % :notes labels))

(defroutes routes
  (user-form (fn [request form] (views/layout form)))
  (GET "/" [] (views/home))
  (route/not-found "<h1>Not Found</h1>"))

(def application-routes (-> routes
                            wrap-stateful-session
                            (wrap-file "public")))

(defn run []
  (run-jetty (var application-routes) {:join? false :port 8080}))
